__all__ = ['BioSequence']

_geneticCode = {'GCU':'A', 'GCC':'A', 'GCA':'A', 'GCG':'A', 
                 'UUA':'L', 'UUG':'L', 'CUU':'L', 'CUC':'L', 'CUA':'L', 'CUG':'L', 
                 'CGU':'R', 'CGC':'R', 'CGA':'R', 'CGG':'R', 'AGA':'R', 'AGG':'R', 
                 'AAA':'K', 'AAG':'K', 
                 'AAU':'N', 'AAC':'N', 
                 'AUG':'M', 
                 'GAU':'D', 'GAC':'D', 
                 'UUU':'F', 'UUC':'F', 
                 'UGU':'C', 'UGC':'C', 
                 'CCU':'P', 'CCC':'P', 'CCA':'P', 'CCG':'P', 
                 'CAA':'Q', 'CAG':'Q', 
                 'UCU':'S', 'UCC':'S', 'UCA':'S', 'UCG':'S', 'AGU':'S', 'AGC':'S', 
                 'GAA':'E', 'GAG':'E', 
                 'ACU':'T', 'ACC':'T', 'ACA':'T', 'ACG':'T', 
                 'GGU':'G', 'GGC':'G', 'GGA':'G', 'GGG':'G', 
                 'UGG':'W', 
                 'CAU':'H', 'CAC':'H', 
                 'UAU':'Y', 'UAC':'Y', 
                 'AUU':'I', 'AUC':'I', 'AUA':'I', 
                 'GUU':'V', 'GUC':'V', 'GUA':'V', 'GUG':'V', 
                 'UAA':'.', 'UGA':'.', 'UAG':'.' }

_complementaryBases = {'A':'T', 'C':'G', 'G':'C', 'T':'A'}

class BioSequence:
    ''' This class implements a nucleotide sequence'''
 
    def __init__(self, name = '', sequence = '', sequenceType = 'DNA'):
        ''' Class constructor '''

        # Here we define instance attributes
        self.name = name
        self.sequence = sequence.strip()
        self.size = len(self.sequence)
        self.sequenceType = sequenceType
        self.__check()

    def __str__(self):
        unit = {'RNA': 'nt', 'DNA':'nt', 'Protein':'aa'}
        string = '%s (%s %s %s):\n'%(self.name, self.sequenceType, str(self.size).rjust(4), unit[self.sequenceType])
        string += ''.join(self.sequence[i:i+60] + '\n' for i in xrange(0, len(self.sequence), 60)) 
        return string

    def __add__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.name, self.sequence + other.sequence, self.sequenceType)

    def append(self, appendedBioSequence):
        if type(appendedBioSequence) == str:
            self.sequence += appendedBioSequence
        elif type(appendedBioSequence) == type(self):
            self.sequence += appendedBioSequence.sequence
        else:
            print "Only text strings or other sequences can be appended"
        self.size = len(self.sequence)
        self.__check()

    def prepend(self, prependedBioSequence):
        if type(prependedBioSequence) == str:
            self.sequence = prependedBioSequence + self.sequence
        elif type(appendedBioSequence) == type(self):
            self.sequence = prependedBioSequence.sequence + self.sequence
        else:
            print "Only text strings or other sequences can be prepended"
        self.size = len(self.sequence)
        self.__check()

    def __check(self):
        ''' Checks if the sequence is valid according to the sequence type '''
        pass

    def complementary(self):
        ''' Creates the complementary sequence'''
        complementaryStr = ''
        for i in range(self.size):
            complementaryStr += _complementaryBases[s1.sequence[-i]]
        return self.__class__(self.name, complementaryStr, self.sequenceType)

    def toRNA(self):
        if self.sequenceType == 'RNA':
            pass
        elif self.sequenceType == 'DNA':
            self.sequence = self.sequence.replace('T', 'U')
            self.sequenceType = 'RNA'
        else:
            print "Sequence cannot be converted to RNA"

    def toDNA(self):
        ''' Convert a RNA sequence to a DNA sequence
        '''
        if self.sequenceType == 'RNA':
            self.sequence = self.sequence.replace('U', 'T')
            self.sequenceType = 'DNA'
        elif self.sequenceType == 'DNA':
            pass
        else:
            print "Sequence cannot be converted to DNA"

    def toProtein(self, readingFrame = 0):
        ''' Convert a DNA or RNA sequence to protein using the Human Genetic Code.
            if a reading frame is specified, it will be used 
            '''
        if self.sequenceType == 'RNA':
            rnaSeq = self.sequence[readingFrame:]
            length = len(rnaSeq)
            print length
            proteinSeq = ''.join([_geneticCode[rnaSeq[i:i+3]] for i in xrange(readingFrame, length - length % 3, 3)])
            self.sequence = proteinSeq
            self.sequenceType = 'Protein'
            self.size = len(proteinSeq)
        elif self.sequenceType == 'DNA':
            self.toRNA()
            self.toProtein()
        elif self.sequenceType == 'Protein':
            pass
        else:
            print 'Sequence cannot be converted to protein'


if __name__ == "__main__":
    
    s1 = BioSequence('BRCA2_01',
                        'TACCAAGCCCTGCGGAGCAAGGTACCTCACACTTCATGAGCGAGTTAAGATGGGTTTCAC'+
                        'AATTTTTCAAGCAAGGAAACGGGCTCGGAGGTCTTGAACACCTGCTACCCAATAGCAGAA'+
                        'CAGCTACTGGAACTAAAATCCTCTGATTTCAAATAACAGCCCCGCCCACTACCACTAAGT'+
                        'GAAGTCATCCACAACCACACACCGACCACTCTAAGCTTTTGTAAGATCGGCTCGCTTTGG')

    s2 = BioSequence('BRCA2_02',
                        'AAGGCTGGGCGTGGTGGCTCACGCCTGTAATCCCAGCACTTTGGGAGGCCGAGGTGGGCG'+
                        'GATCACTTGAGGCCAGAAGTTTGAGACCAGCCTGGCCAACATGGTGAAACCCTATCTCTA'+
                        'CTAAAAATACAAAAAATGTGCTGCGTGTGGTGGTGCGTGCCTGTAATCCCAGCTACACGG'+
                        'GAGGTGGAGGCAGGAGAATCGCTTGAACCCTGGAGGCAGAGGTTGCAGTGAGCCAAGATC'+
                        'ATGCCACTGCACTCTAGCCTGGGCCACATAGCATGACTCTGTCTCAAAACAAACAAACAA'+
                        'ACAAAAAACTAAGAATTTAAAGTTAATTTACTTAAAAATAATGAAAGCTAACCCATTGCA')  
    
    print s1
    print s2

    s1.append('TA')
    print s1

    s1.prepend('CGT')
    print s1

    s1.complementary()
    print s1

    s3 = s1 + s2
    s3.name = s1.name + '_' + s2.name
    print s3

    s3.toRNA()
    print s3
    
    s3.toProtein()
    print s3
    s1.toProtein(1)
    print s1


