# -*- coding: utf-8 -*-
import Parameters
from pylab import *

#################################
# Defining analytical functions #
#################################
def Gauss_1D(p):
    return 1.0/(sqrt(4.0*pi*p.D*(p.t-p.tp)))*exp(-(p.xs-p.c)*(p.xs-p.c)/(4.0*(p.t-p.tp)*p.D))

#########################
# FFT related functions #
#########################
def fft_1D(Data):
    return fftshift( fft(Data) )

def ifft_1D(Data):
    return ifft( fftshift(Data) ).astype(float)

def Freq_1D(n,dx):
    Fs=1.0/dx         # Sampling frequency
    f=fftshift(Fs*fftfreq(n))
    return f

def fft_2D(psi):
    return fftshift( fft2(psi) )

def ifft_2D(Data):
    return ifft2( fftshift(Data) ).astype(float)

def Freq_2D(nx,ny,dx):
    Fs=1.0/dx         # Sampling frequency
    lx=fftshift(Fs*fftfreq(nx))
    ly=fftshift(Fs*fftfreq(ny))
    fx=zeros( (nx,ny) )
    fy=zeros( (nx,ny) )
    for i in range(nx):
	for j in range(ny):
	      fx[i,j]=lx[i]
	      fy[i,j]=ly[j]
    
    return fx,fy

##############
# Euler step #
##############
def Euler_Step(psi,p,Change_t=True):
    Command="f="+p.rhs
    exec(Command)
    psi=psi+p.dt*f
    if Change_t == True:
       p.it=p.it+1
       p.t=p.t+p.dt
    return psi
#########################
# Initial conditions 2D #
#########################
def IC_2D(N,kx,xs):
    noise=2.0e-1
    New_Data=noise*randn(N,N)+sin(kx*xs)
    return New_Data

#####################
# Ploting functions #
#####################
def Plot_1D(psi,p,Add_Gaussian=True):
    if hasattr(p,"Init") == False:
       ion()
       fig = figure(1)
       sub = subplot(111)
       if Add_Gaussian == True:
          lines = plot(p.xs,psi,'b-',p.xs,Gauss_1D(p),'r-')
          legend( (lines[0], lines[1]),
	          ('Numeric sol.', 'Analytical sol.'),
		  'upper center',shadow=True)

       else:
          lines = plot(p.xs,psi,'b-')
	  legend(lines,'Numeric sol.','upper center',shadow=True)
       xlabel('x')
       ylabel('f(x)')
       draw()
       sub.axis([p.xmin,p.xmax,p.ymin,p.ymax])
       axis([p.xmin, p.xmax, p.xmin, p.ymax])
       p('lines',lines)
       p("Init",True)
    p.lines[0].set_data(p.xs,psi)
    if Add_Gaussian == True:
       p.lines[1].set_data(p.xs,Gauss_1D(p))
    if p.Autoscale == True:
       ymax=psi.max()
       if Add_Gaussian == True:
	  print ">",ymax
          ymax=max(Gauss_1D(p).max(),ymax)
	  print "<",ymax
       axis([p.xmin, p.xmax,0.0,ymax])
    draw()
    if p.Save == True:
       pylab.savefig('Fig_%05.2f.eps' % (p.t) )
    print "Time: %04.4f/%04.4f" % (p.t,p.tmax)
    return

def Plot_2D(psi,p):
    if hasattr(p,"Init") == False:
       ion()
       fig = figure(1)
       sub = subplot(111)
       im = imshow(psi,vmin=p.zmin,vmax=p.zmax)
       cbar = fig.colorbar(im)
       axis('off')
       p('im',im)
       p('sub',sub)
       p("Init",True)
    p.im.set_data(psi)
    if  psi.max()-psi.min() > 1.0e-1:
        p.im.set_clim(vmin=psi.min(),vmax=psi.max())
    else:
	exit()
    draw()
    if p.Save == True:
       savefig('Fig_%06.2f.eps' % (p.t) )
    print "Time: %04.4f/%04.4f" % (p.t,p.tmax)
    return
