#!/usr/bin/python
# -*- coding: utf-8 -*-

from Functions import *

###############
# Definitions #
###############
p=Parameters.Parameters()

#######################
# Defining Parameters #
#######################
p('dt',0.3)
p('it',0)
p('t',0.0)
p('zmin',-0.2)
p('zmax',-0.2)
p('Save',False)

#Instability_Type = "Eckhaus"
#Instability_Type = "Stripes"
Instability_Type = "Hexagons"
#Instability_Type = "Zig-Zag"

p('rhs','p.g*psi**2-psi**3')

N=128
xmin=0.0
xmax=20.0*pi
ymin=0.0
ymax=xmin
tmax=100.0

if Instability_Type == "Eckhaus":
   N=N*2
   xmax=ymax=xmax*2
   tmax=35.0

dx=(xmax-xmin)/N

##################
# Defining Space #
##################
xs=linspace(0,xmax,N)
p('xs',xs); p('ys',xs)

#######################
# Equation parameters #
#######################
if Instability_Type == "Eckhaus":
   kx=1.178; p.epsilon=0.3; p.g=0.0

if Instability_Type == "Stripes":
   kx=0.0; p.epsilon=0.3; p.g=0.0

if Instability_Type == "Hexagons":
   kx=0.5; p.epsilon=0.1; p.g=1.0 # from stripes
   kx=0.0; p.epsilon=0.1; p.g=1.0 # from noise

if Instability_Type == "Zig-Zag":
   kx=0.88357; p.epsilon=0.3; p.g=0.0

psi=IC_2D(N,kx,xs)

p('tmax',tmax/p.epsilon)
tout=p.tmax/100
itout=int(tout/	p.dt)+1

###################
# Initialize Plot #
###################
Plot_2D(psi,p)

##############
# Wavenumber #
##############
(fx,fy)=Freq_2D(N,N,dx)
p('kx',fx*2.0*pi)
p('ky',fy*2.0*pi)


##############
# Defining q #
##############
p('q',1.0/(1.0-p.dt*(p.epsilon-(1.0-(p.kx**2+p.ky**2))**2)))

#############################
# Saving/Reading parameters #
#############################
#File='Parameters_2D_%s.dat' % (Instability_Type)
#p.Write_To_File(File)
#p.Read_From_File(File)

while p.it*p.dt < p.tmax:
      psi=ifft_2D(fft_2D(psi)*p.q)
      psi=Euler_Step(psi,p)
      if (p.it%itout) == 0:
          Plot_2D(psi,p)
